#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file WiFiNeighborNodeObserver.py
 @brief This RTC observes Wi-Fi neighbor nodes information.
 @date $Date$

 @author 株式会社 東日本計算センター

 プロジェクト方針決定後に記載

"""
import sys
import copy
import uuid
from logging import getLogger, StreamHandler
from logging import INFO as LOG_LV

# Import RTM module
import RTC
import OpenRTM_aist
import WiFiNodeIF

import neighbornodelist

sys.path.append(".")

# Setup logger
logger = getLogger(__name__)
handler = StreamHandler()
handler.setLevel(LOG_LV)
logger.setLevel(LOG_LV)
logger.addHandler(handler)

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
wifineighbornodeobserver_spec = ["implementation_id", "WiFiNeighborNodeObserver",
                                 "type_name",         "WiFiNeighborNodeObserver",
                                 "description",
                                 "This RTC observes Wi-Fi neighbor nodes information.",
                                 "version",           "0.1",
                                 "vendor",            "EAC",
                                 "category",          "Network",
                                 "activity_type",     "STATIC",
                                 "max_instance",      "1",
                                 "language",          "Python",
                                 "lang_type",         "SCRIPT",
                                 "conf.default.node_id", "auto",
                                 "conf.__widget__.node_id", "text",
                                 ""]
# </rtc-template>


##
# @class WiFiNeighborNodeObserver
# @brief This RTC observes Wi-Fi neighbor nodes information.
#
# 別紙"機能仕様書_WiFiNeighborNodeObserverRTC"参照
#
# 別紙"機能仕様書_WiFiNeighborNodeObserverRTC"参照
#
# 別紙"機能仕様書_WiFiNeighborNodeObserverRTC"参照
#
# 別紙"機能仕様書_WiFiNeighborNodeObserverRTC"参照
#
class WiFiNeighborNodeObserver(OpenRTM_aist.DataFlowComponentBase):
    ##
    # @brief constructor
    # @param manager Maneger Object
    #

    def __init__(self, manager):
        print(manager.__class__)
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_WiFiNeighborNodeListOut = WiFiNodeIF.TimedNeighborNodeList(RTC.Time(0, 0),
                                                                           0,
                                                                           [])
        """
        WiFi近隣ノード情報を出力する
         - Type: WiFiNodeIF::TimedNeighborNodeList
         - Number: 動的確保
         - Frequency: コンポーネント実行周期と同様
         - Operation Cycle: コンポーネント実行周期と同様
        """
        self._WiFiNeighborNodeListOutOut = OpenRTM_aist.OutPort("WiFiNeighborNodeListOut",
                                                                self._d_WiFiNeighborNodeListOut)

        # 近隣ノード情報コンテナ生成
        try:
            print("Create object")
            self._neighbornodelist = neighbornodelist.WiFiNeighborNodeList.create_object()
        except:
            return RTC.RTC_ERROR

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">
        """
        コンポーネントノードID。
        WiFiNeighborNodeList.idに使用される。
         - Name:  node_id node_id
         - DefaultValue: DEFAULT_NODE_ID
         - Constraint: configuration fileに以下のように事前に設定する事
                       設定されていない場合、MACアドレスが代わりに使用される。
                       conf.default.node_id: drone_01
        """
        self.DEFAULT_NODE_ID = "auto"
        self._node_id = [self.DEFAULT_NODE_ID]

        # </rtc-template>

    ##
    # RTC初期化
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    def onInitialize(self):
        # Bind variables and configuration variable
        self.bindParameter("node_id", self._node_id, self.DEFAULT_NODE_ID)

        # Set InPort buffers

        # Set OutPort buffers
        self.addOutPort(
            "WiFiNeighborNodeListOut", self._WiFiNeighborNodeListOutOut)

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        # Set node id
        self._set_node_id()

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The finalize action (on ALIVE->END transition)
    #    # formaer rtc_exiting_entry()
    #    #
    #    # @return RTC::ReturnCode_t
    #
    #    #
    # def onFinalize(self):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The activated action (Active state entry action)
    #    # former rtc_active_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onActivated(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The deactivated action (Active state exit action)
    #    # former rtc_active_exit()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onDeactivated(self, ec_id):
    #
    #    return RTC.RTC_OK

    ##
    # Wi-Fi近隣ノード情報を更新してOutポートバッファへ書き出す。
    # 経路情報に変化が無くても書き出す(Observer自身の生存報告を兼ねているため)。
    # 変化があった場合は、ログファイルに記録する。
    #
    # The execution action that is invoked periodically
    # former rtc_active_do()
    #
    # @param ec_id target ExecutionContext Id
    #
    # @return RTC::ReturnCode_t
    #
    def onExecute(self, ec_id):

        try:
            # logger.info("Start Scan")
            self._neighbornodelist.scan()
        except:
            return RTC.RTC_ERROR

        OpenRTM_aist.setTimestamp(self._d_WiFiNeighborNodeListOut)

        self._set_node_id()  # 動的なID変更に対応するため

        result = self._neighbornodelist.node_list

        if self._d_WiFiNeighborNodeListOut.data != result:
            del self._d_WiFiNeighborNodeListOut.data[:]

            self._d_WiFiNeighborNodeListOut.data = copy.deepcopy(result)

            logger.info("updated Wi-Fi neighbor node list:")
            logger.info(self._d_WiFiNeighborNodeListOut.tm)
            for data in self._d_WiFiNeighborNodeListOut.data:
                logger.info(data)

        self._WiFiNeighborNodeListOutOut.write()

        return RTC.RTC_OK

    ##
    # エラーログ出力
    #
    # The aborting action when main logic error occurred.
    # former rtc_aborting_entry()
    #
    # @param ec_id target ExecutionContext Id
    #
    # @return RTC::ReturnCode_t
    #
    def onAborting(self, ec_id):
        logger.error("ERROR: detected an Aborting state")

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The error action in ERROR state
    #    # former rtc_error_do()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onError(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The reset action that is invoked resetting
    #    # This is same but different the former rtc_init_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onReset(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    ##
    # @brief 自分のノードIDを設定する。
    #
    # 自分のノードIDを設定する。
    # confファイルで指定されているノードIDを使用する。
    # confファイルで指定されていなければ、uuid.getnode()で自動生成する(10新表記48bit符号無整数 from MAC Address)。
    # Active動作中に自ノードIDが変更された場合、結果的に変更後の値がセットされる。
    #
    # @param self
    def _set_node_id(self):
        # result = hex(uuid.getnode()).upper()
        # result[2:4] + ":" result[4:6] + ～略～ これで16進MACアドレス表記になる
        
        self._self_node_id = self._node_id[0]
        if self._self_node_id == self.DEFAULT_NODE_ID:
            self._self_node_id = str(uuid.getnode())

        if self._d_WiFiNeighborNodeListOut.id != self._self_node_id:
            self._d_WiFiNeighborNodeListOut.id = self._self_node_id
            logger.info("new node id: " + self._d_WiFiNeighborNodeListOut.id)

            if self._self_node_id == str(uuid.getnode()):
                logger.warning(
                    "WARNING: node id should be set in configuration file")
                logger.warning("WARNING: MAC address is used instead")


def WiFiNeighborNodeObserverInit(manager):
    profile = OpenRTM_aist.Properties(
        defaults_str=wifineighbornodeobserver_spec)
    manager.registerFactory(profile,
                            WiFiNeighborNodeObserver,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    WiFiNeighborNodeObserverInit(manager)

    # Create a component
    comp = manager.createComponent("WiFiNeighborNodeObserver")


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager()

if __name__ == "__main__":
    main()
